@@ -1,4 +1,4 @@
-# $Header: /home/jcb/MahJong/newmj/RCS/proto-decode-msg.pl,v 12.0 2009/06/28 20:43:13 jcb Rel $
+# $Header: /home/jcb/newmj/RCS/proto-decode-msg.pl,v 11.0 2001/05/17 18:23:12 jcb Rel $
 # proto-decode-msg.pl
 # script to generate body of decode_[cp]msg function in protocol.c
 # it is not defensively programmed!
@@ -23,25 +23,13 @@ $debug = $ENV{'DEBUG'};
 # are we doing cmsg or pmsg ?
 if ( $ARGV[0] eq '-cmsg' ) {
   $L = 'C' ; $l = 'c';
-  $msgtype = "Controller";
-  $infile = "protocol.h";
 } elsif ( $ARGV[0] eq '-pmsg' ) {
   $L = 'P' ; $l = 'p' ;
-  $msgtype = "Player";
-  $infile = "protocol.h";
-} elsif ( $ARGV[0] eq '-mcmsg' ) {
-  $L = 'MC' ; $l = 'mc' ;
-  $msgtype = "MController";
-  $infile = "mprotocol.h";
-} elsif ( $ARGV[0] eq '-mpmsg' ) {
-  $L = 'MP' ; $l = 'mp' ;
-  $msgtype = "MPlayer";
-  $infile = "mprotocol.h";
 } else {
   die("No function argument");
 }
 
-open(STDIN,"<$infile") or die("No infile");
+open(STDIN,"<protocol.h");
 
 open(STDOUT,">dec_${l}msg.c");
 
@@ -54,7 +42,7 @@ open(STDOUT,">dec_${l}msg.c");
 #
 #   m->type = CMsgPlayerPungs;
 #
-#   if ( sscanf(s,"%d%n",&an_int,&n) == 0 ) { warn("protocol error\n"); return (CMsgMsg *)0; }
+#   if ( sscanf(s,"%d %n",&an_int,&n) == 0 ) { warn("protocol error\n"); return (CMsgMsg *)0; }
 #   m->id = an_int;
 #   s += n;
 #
@@ -102,59 +90,65 @@ while ( ! eof(STDIN) ) {
       chop;
       s/;.*$// ; # junk all but type and name
       if ( s/^\s*int\s+// ) {
-	print '    if ( sscanf(s,"%d%n",&an_int,&n) == 0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
+	print '    if ( sscanf(s,"%d %n",&an_int,&n) == 0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
 	print "    m->$_ = an_int;\n";
-	print "    s += n;\n";
-	print "    while ( isspace(*s) ) s++;\n\n";
+	print "    s += n;\n\n";
 	next;
       }
       if ( s/^\s*bool\s+// ) {
-	print '    if ( sscanf(s,"%d%n",&an_int,&n) == 0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
+	print '    if ( sscanf(s,"%d %n",&an_int,&n) == 0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
         print '    if ( an_int < 0 || an_int > 1 ) { warn("protocol error\n") ; return (' . $L . 'MsgMsg *)0; }', "\n";
 	print "    m->$_ = an_int;\n";
-	print "    s += n;\n";
-	print "    while ( isspace(*s) ) s++;\n\n";
+	print "    s += n;\n\n";
 	next;
       }
       if ( s/^\s*PlayerOption\s+// ) {
-	print '    if ( sscanf(s,"%31s%n",little_string,&n) ==0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
+	print '    if ( sscanf(s,"%s %n",little_string,&n) ==0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
 	print "    m->$_ = player_scan_PlayerOption(little_string);\n";
-	print "    s += n;\n";
-	print "    while ( isspace(*s) ) s++;\n\n";
+	print "    s += n;\n\n";
 	next;
       }
       if ( s/^\s*TileWind\s+// ) {
-	print '    if ( sscanf(s,"%c%n",&a_char,&n) == 0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
+	print '    if ( sscanf(s,"%c %n",&a_char,&n) == 0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
 	print "    m->$_ = letterwind(a_char);\n";
-	print "    s += n;\n";
-	print "    while ( isspace(*s) ) s++;\n\n";
+	print "    s += n;\n\n";
+	next;
+      }
+      if ( s/^\s*Tile tiles\[13\]// ) {
+	# this is a special case
+	print "    if ( ! m->hidden ) {\n";
+	print "      for ( i = 0 ; i < 13 ; i++ ) {\n";
+        print '        if ( sscanf(s,"%2s %n",little_string,&n) == 0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
+	print "        s += n;\n\n";
+	print "        m->tiles[i] = tile_decode(little_string);\n";
+	print "        if ( ! m->tiles[i] ) { warn(\"protocol error\\n\"); return (${L}MsgMsg *)0; }\n";
+        print "      }\n";
+        print "    }\n\n";
 	next;
       }
       if ( s/^\s*Tile\s+// ) {
-	print '    if ( sscanf(s,"%2s%n",little_string,&n) == 0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
+	print '    if ( sscanf(s,"%2s %n",little_string,&n) == 0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
 	print "    m->$_ = tile_decode(little_string);\n";
 	print "    if ( m->$_ == ErrorTile ) { warn(\"protocol error\\n\"); return (${L}MsgMsg *)0; }\n";
-	print "    s += n;\n";
-	print "    while ( isspace(*s) ) s++;\n\n";
+
+	print "    s += n;\n\n";
 	next;
       }
       if ( s/^\s*word(\d+)\s+// ) {
 	$wordlen = $1;
-	print '    if ( sscanf(s,"%',$wordlen,'s%n",little_string,&n) == 0 ) { warn("protocol error\n");  return (' . $L . 'MsgMsg *)0; }', "\n";
+	print '    if ( sscanf(s,"%',$wordlen,'s %n",little_string,&n) == 0 ) { warn("protocol error\n");  return (' . $L . 'MsgMsg *)0; }', "\n";
 	print "    strmcpy(m->$_,little_string,$wordlen);\n";
-	print "    s += n;\n";
-	print "    while ( isspace(*s) ) s++;\n\n";
+	print "    s += n;\n\n";
 	next;
       }
       if ( s/^\s*ChowPosition\s+// ) {
-	# the 31 field width is for safety. This string should always
+	# the 32 field width is for safety. This string should always
 	# be lower, middle or upper!
 	# Argh. This code does not defend against protocol errors.
 	# FIXME
-	print '    if ( sscanf(s,"%31s%n",little_string,&n) == 0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
+	print '    if ( sscanf(s,"%32s %n",little_string,&n) == 0 ) { warn("protocol error\n"); return (' . $L . 'MsgMsg *)0; }', "\n";
 	print "    m->$_ = string_cpos(little_string);\n";
-	print "    s += n;\n";
-	print "    while ( isspace(*s) ) s++;\n\n";
+	print "    s += n;\n\n";
 	next;
       }
       if ( s/^\s*char\s+\*\s*// ) {
@@ -163,11 +157,7 @@ while ( ! eof(STDIN) ) {
 	print "    else {\n";
 	print "      m->$_ = (char *)malloc(strlen(s)+1);\n";
 	print "      if ( ! m->$_ ) { warn(\"malloc failed\\n\"); return (${L}MsgMsg *)0; }\n";
-	# we now need to do a little backslash unescaping.
-        # At this level of protocol, all we do is say: if arg starts
-	# with \, remove it - any initial backslash must be there
-	# to protect the next character
-	print "      strcpy(m->$_,s+(s[0] == '\\\\'));\n";
+	print "      strcpy(m->$_,s);\n";
 	print "    }\n";
 	next;
       }
