@@ -1,4 +1,4 @@
-# $Header: /home/jcb/MahJong/newmj/RCS/proto-encode-msg.pl,v 12.0 2009/06/28 20:43:13 jcb Rel $
+# $Header: /home/jcb/newmj/RCS/proto-encode-msg.pl,v 11.1 2001/08/05 12:34:10 jcb Rel $
 # proto-encode-msg.pl
 # generate the protocol conversion switch for c/pmsgs
 # Also, generate in cmsg_union.h a definition of a 
@@ -25,25 +25,13 @@ $debug = $ENV{'DEBUG'};
 # are we doing cmsg or pmsg ?
 if ( $ARGV[0] eq '-cmsg' ) {
   $L = 'C' ; $l = 'c';
-  $msgtype = "Controller";
-  $infile = "protocol.h";
 } elsif ( $ARGV[0] eq '-pmsg' ) {
   $L = 'P' ; $l = 'p' ;
-  $msgtype = "Player";
-  $infile = "protocol.h";
-} elsif ( $ARGV[0] eq '-mcmsg' ) {
-  $L = 'MC' ; $l = 'mc' ;
-  $msgtype = "MController";
-  $infile = "mprotocol.h";
-} elsif ( $ARGV[0] eq '-mpmsg' ) {
-  $L = 'MP' ; $l = 'mp' ;
-  $msgtype = "MPlayer";
-  $infile = "mprotocol.h";
 } else {
   die("No function argument");
 }
 
-open(IN,"<$infile") or die("No infile");
+open(IN,"<protocol.h");
 
 open(STDOUT,">enc_${l}msg.c");
 open(UNION,">${l}msg_union.h");
@@ -53,7 +41,7 @@ print UNION
 "typedef union _${L}MsgUnion {
   /* Note that this type field relies on the fact that all
      messages have type as their first field */\n";
-print UNION $msgtype, "MsgType type;\n";
+print UNION +($L eq 'P' ? "Player" : "Controller"), "MsgType type;\n";
 
 print SIZE
 "static int ${l}msg_size[] = {\n";
@@ -94,13 +82,13 @@ while ( ! eof(IN) ) {
 	print "    if ( m->$_ < 0 || m->$_ > 1 ) { warn(\"bad boolean in message, assuming TRUE\"); badfield = 1 ; };\n";
 	$args .= ", (badfield ? 1 : m->$_)";
       } elsif ( s/^\s*char \*// ) {
-	s/;.*//;
-	# if the argument happens to start with a space or backslash, then
+	# if the argument happens to start with a space, then
 	# we need to escape it with a backslash
 	$format .= " %s%s";
+	s/;.*//;
 	# if the argument is null, don't pass it to printf,
 	# but pass the empty string instead
-	$args .= ",(m->$_ && (m->$_\[0] == ' ' || m->$_\[0] == '\\\\')) ? \"\\\\\" : \"\", m->$_ ? m->$_ : \"\" ";
+	$args .= ",(m->$_ && m->$_\[0] == ' ') ? \"\\\\\" : \"\", m->$_ ? m->$_ : \"\" ";
 	# and flag this type:
 	$charstar{$type} = $_; # see below ...
       } elsif ( s/^\s*word\d+\s+// ) {
@@ -115,6 +103,12 @@ while ( ! eof(IN) ) {
 	$format .= " %s";
 	s/;.*//;
 	$args .= ", player_print_PlayerOption(m->$_)";
+      } elsif ( s/^\s*Tile tiles\[13\]// ) {
+	# special cased
+	$dofinish = 0;
+	print "      sprintf(buf,\"$format\"$args);\n";
+	print '      if ( !m->hidden) for ( i = 0; i < 13; i++ ) { add(" "); add(tile_code(m->tiles[i])); }', "\n";
+	print "      add(\"\\r\\n\");\n    }\n    break;\n";
       } elsif ( s/^\s*Tile\s+// ) {
 	$format .= " %s";
 	s/;.*//;
@@ -132,17 +126,7 @@ while ( ! eof(IN) ) {
     }
     next if ! $dofinish;
     $format .= '\r\n';
-    print "      while (1) {\n";
-    print "        int size = snprintf(buf,buf_size,\"$format\"$args);\n";
-    print "        if ((size >= 0)&&(size < (int)buf_size))\n";
-    print "          break;\n";
-    print "        buf_size = (size >= 0)?((size_t)size+1):(buf_size > 0)?(buf_size*2):1024;\n";
-    print "        buf = realloc(buf, buf_size);\n";
-    print "        if (!buf) {\n";
-    print "          perror(\"encode_" . $l . "msg\");\n";
-    print "          exit(-1);\n";
-    print "        }\n";
-    print "      }\n";
+    print "      sprintf(buf,\"$format\"$args);\n";
     print "    }\n";
     print "    break;\n";
   }
@@ -164,7 +148,9 @@ print UNION "} ${L}MsgUnion;\n";
 print SIZE 
 "};
 
-int ${l}msg_size_of(",$msgtype,"MsgType t) { return ${l}msg_size[t]; }\n\n";
+int ${l}msg_size_of(",
+($L eq 'P' ? "Player" : "Controller"),
+  "MsgType t) { return ${l}msg_size[t]; }\n\n";
 
 # While we're at it, we'll also generate functions to do
 # a deep copy (malloc'ing all space), and to deep free
